%========================== PlotDistResults.m =============================
% This script loads percept duration data and plots various descriptive 
% statistics for each subject.
%==========================================================================

BinSize     = 0.5;                      % 
alpha       = 0.05;                     %
UpperLimit  = 30;                       % Upper threshold for outliers (seconds)
LowerLimit  = 0.3;                      % Lower threshold for outliers (seconds)
XLabel      = 'Percept Duration (s)';   


ResultsFile = 'AllPerceptDurations.mat';   
load(ResultsFile);                      % Load all data
Durations = RSDurations;                % Analyse rotating sphere (RS) task data
Durations(Durations>UpperLimit) = nan;  % Remove outlier durations
Durations(Durations<LowerLimit) = nan;
PatientIDs = SubjectIDs(1:5);           
ControlIDs = SubjectIDs(~strmatch('Ctrl', SubjectIDs));


AllxLims = [20,20,30,30,30,30,30,40, 40, 40, 40];

p = 0;
AllStats = nan(numel(SubjectIDs), 8);
AllLognStat= nan(numel(SubjectIDs), 10);
AllGamStat= nan(numel(SubjectIDs), 10);
figure('name','Percept duration distributions');
for S = 1:numel(SubjectIDs)
    p = p+1;
    f(p) = subplot(numel(SubjectIDs),2,p);  
    DurData = Durations(:,S);
    DurData(isnan(DurData))= [];
    
    bins = 0:BinSize:max(DurData);
    bins = 0:BinSize:AllxLims(S);
    [n,xout] = hist(DurData, bins);                                       
    n = n/(numel(DurData)*BinSize);                                       % For n = probability of a given phase duration 
    h1 = bar(xout,n, 'histc');                                             
    box off;
    set(gca,'TickDir','out');
    set(h1,'facecolor','w');
    set(gca,'fontsize',10);  
    
    if S ==numel(SubjectIDs)
        xlabel(XLabel, 'FontSize', 12)                                          % Add x- and y-axis titles
%         linkaxes(f,'x');
%         set(gca,'xlim',[0 50]);
%      	set(gca, 'ylim',[0 0.2]);
    end
    ylabel('Frequency', 'FontSize', 12)
    title(sprintf('Subject %s',SubjectIDs{S}), 'FontSize', 14, 'FontWeight','bold')
    Ylim = get(gca, 'ylim');
    Xlim = get(gca, 'xlim');
    Xticks = 0:0.02:Xlim(2);
    hold on;
    set(gca,'xlim',[0 AllxLims(S)]);


    %================== Fit LOGNORMAL duration distribution to data
    [parmhat, parmci] = lognfit(DurData);        	% 'pci' are confidence intervals (lower bound; upper bound)
    LogMu = parmhat(1);                            	% 'parmhat' returns estimated mu and sigma parameters
    LogSigma = parmhat(2);
    [LogMean,LogVar] = lognstat(LogMu,LogSigma);   	% obtain mean and variance for lognormal distribution
    logX = 0:0.02:Xlim(2);
    logPDF = lognpdf(logX,LogMu,LogSigma);
    logCDF = logncdf(logX,LogMu,LogSigma);

    %================== Fit GAMMA duration distribution to data
    [phat,pci] = gamfit2(DurData, alpha);         % 'phat' returns maximum likelihood estimates (MLEs), 'pci' are confidence intervals (lower bound; upper bound)
    GamK = phat(1);                                 % shape parameter (k)
    GamLambda = phat(2);                            % scale parameter (theta) 
    [GamMean,GamVar] = gamstat(GamK,GamLambda);     % obtain mean and variance for gamma distribution
    gamX = 0:0.02:Xlim(2);
    gamPDF = gampdf(gamX,GamK, GamLambda);
    gamCDF = gamcdf(gamX,GamK, GamLambda);

    %================== Fit GAMMA RATE distribution to data
    Rates = 1/DurData;
    [Rphat,Rpci] = gamfit2(Rates, alpha);           % Fit gamma distribution to SWITCH RATE data
    RGamK = Rphat(1);                               % shape parameter (k)
    RGamLambda = Rphat(2);                          % scale parameter (lambda) 
    [RGamMean,RGamVar] = gamstat(RGamK,RGamLambda); % obtain mean and variance for gamma rate distribution
    RgamX = 0:0.02:Xlim(2); 
    RategamPDF = gampdf(RgamX,RGamK, RGamLambda);
    RgamPDF = zeros(numel(RgamX),1);
    RgamCDF = zeros(numel(RgamX),1);
    for n = 1:numel(RgamX)
        X(n) = 1/(RgamX(n));
        RgamPDF(n) = (1/((RGamLambda^RGamK)*gamma(RGamK))) * X(n)^(RGamK-1) * exp(-X(n)/RGamLambda);
        RgamPDF(n) = RgamPDF(n)/(RgamX(n))^2;
        RgamCDF(n) = sum(RgamPDF);
    end


    %================== Plot all PDF fits
    PDF = plot(logX,logPDF, '-r', gamX,gamPDF, '-b', RgamX,RgamPDF, '-g', 'LineWidth', 2);
    hold on;
    Median = nanmedian(DurData);
    MedDev = mad(DurData,1);
    CI = prctile(DurData,[5, 32, 68, 95]);
    YLims = get(gca,'ylim');
    
    CIh = patch([CI(1),CI(1),CI(2),CI(2)],[YLims, YLims([2,1])], 'r', 'EdgeColor','none');
    
%     MadH = patch([Median-MedDev,Median-MedDev,Median+MedDev,Median+MedDev],[YLims, YLims([2,1])], 'r', 'EdgeColor','none');
    MedH = plot(repmat(Median,[1,2]),ylim,'-r','LineWidth', 2);
    
    if S==1
        l = legend(PDF, {'Lognormal', 'Gamma', 'Gamma rate'}, 3);
        set(l, 'Location','NorthEast');
        set(l, 'Box', 'off');
    end

    %================== Add distribution notes
    LogNotes = strcat('Lognormal: \mu = ', num2str(LogMu, '%10.3f'), ', \sigma = ', num2str(LogSigma, '%10.3f'));
    GamNotes = strcat('Gamma duration: r = ', num2str(GamK, '%10.3f'), ', \lambda = ', num2str(GamLambda, '%10.3f'));
    RGamNotes = strcat('Gamma rate: r = ', num2str(RGamK, '%10.3f'), ', \lambda = ', num2str(RGamLambda, '%10.3f'));
    Notes = {LogNotes; GamNotes; RGamNotes};
%     text(Xlim(2)/2,(Ylim(2)-(Ylim(2)/3)), Notes);
    fprintf('\n\n******* Subject %s **********\n', SubjectIDs{S});
    disp(Notes);

    
    %===================== PLOT CUMULATIVE DISTRIBUTION FUNCTIONS (CDFs)
 	p = p+1;
    f(p) = subplot(numel(SubjectIDs),2,p);
    [h,stats] = cdfplot(DurData);                                                   % Plot empirical data CDF
    set(h,'Color','k','LineWidth',1)
    set(gca,'fontsize',10);
    if S == numel(SubjectIDs)
        xlabel('Percept Duration (s)', 'FontSize', 12)           % Add x- and y-axis titles
    end
%     title('CDF', 'FontSize', 12, 'FontWeight','bold')
    hold on;

    CDF = plot(logX,logCDF, '-r', gamX,gamCDF, '-b', RgamX,RgamCDF, '-g');
  	box off;
    set(gca,'TickDir','out');
    
    % CDF = plot(logX,logCDF, '-r', gamX,gamCDF, '-b');                                 % Plot all CDF fits
    set(CDF, 'LineWidth', 2);
    %     l = legend(gca, 'Empirical data', 'Lognormal', 'Gamma', 'Gamma rate', 3);
    if S == 1
        l = legend(gca, 'Empirical data', 'Lognormal', 'Gamma', 3);
        set(l, 'Location','SouthEast');
        set(l, 'Box', 'off');
    end
    grid off;

    %================== Chi square test for goodness of fit for each distribution
    [LognH,LognP,LognStats] = chi2gof(DurData, 'cdf', {'logncdf',LogMu,LogSigma});
    [GamH,GamP,GamStats] = chi2gof(DurData, 'cdf', {'gamcdf',GamK, GamLambda});
    [RGamH,RGamP,RGamStats] = chi2gof(DurData, 'cdf', {'gamcdf',RGamK, RGamLambda});

    %================== Save parameter estimates, confidence intervals and chi statistics to .mat file
    TotalPhases(S) = numel(DurData);
    LogParm = struct('mu', LogMu, 'muConfInt', [parmci(1,1), parmci(2,1)], 'sigma', LogSigma, 'sigConfInt', [parmci(1,2), parmci(2,2)]);
    GamParm = struct('k', GamK, 'kConfInt', [pci(1,1), pci(2,1)], 'lambda', GamLambda, 'lambdaConfInt', [pci(1,2), pci(2,2)]);
    RGamParm = struct('k', RGamK, 'kConfInt', [Rpci(1,1), Rpci(2,1)], 'lambda', RGamLambda, 'lambdaConfInt', [Rpci(1,2), pci(2,2)]);
    LogChi2 = struct('Chi2', LognStats.chi2stat, 'p', LognP, 'N', TotalPhases(S), 'dF', LognStats.df);
    GamChi2 = struct('Chi2', GamStats.chi2stat, 'p', GamP, 'N', TotalPhases(S), 'dF', GamStats.df);
    RGamChi2 = struct('Chi2', RGamStats.chi2stat, 'p', RGamP, 'N', TotalPhases(S), 'dF', RGamStats.df);


    %================== Add statistical notes
    LogNote1 = strcat('Lognormal: \mu = ', num2str(LogMu, '%10.3f'), ', \sigma = ', num2str(LogSigma, '%10.3f'));
    LogNote2 = strcat('\chi^{2} (', num2str(LognStats.df), ', N = ', num2str(TotalPhases(S)), ') = ', num2str(LognStats.chi2stat, '%10.3f'), ', \itp\rm = ',num2str(LognP, '%10.3f'));
    GamNote1 = strcat('Gamma duration: r = ', num2str(GamK, '%10.3f'), ', \lambda = ', num2str(GamLambda, '%10.3f'));
    GamNote2 = strcat('\chi^{2} (', num2str(GamStats.df), ', N = ', num2str(TotalPhases(S)), ') = ', num2str(GamStats.chi2stat, '%10.3f'), ', \itp\rm = ',num2str(GamP, '%10.3f'));
    RGamNote1 = strcat('Gamma rate: r = ', num2str(RGamK, '%10.3f'), ', \lambda = ', num2str(RGamLambda, '%10.3f'));
    RGamNote2 = strcat('\chi^{2} (', num2str(RGamStats.df), ', N = ', num2str(TotalPhases(S)), ') = ', num2str(RGamStats.chi2stat, '%10.3f'), ', \itp\rm = ',num2str(RGamP, '%10.3f'));
    LogNotes = {LogNote1; LogNote2};
    GamNotes = {GamNote1; GamNote2};
    RGamNotes = {RGamNote1; RGamNote2};

    disp(LogNotes)
    disp(GamNotes)
    disp(RGamNotes)
    text(20,0.40, LogNotes, 'EdgeColor','red');
    text(20,0.25, GamNotes, 'EdgeColor','blue');
    text(20,0.10, RGamNotes, 'EdgeColor','green');
    
    
    AllStats(S,:) = [mean(DurData), std(DurData), std(DurData)/sqrt(numel(DurData)), Median, CI];
    AllLognStat(S,:) = [LogMu, parmci(1,1), parmci(2,1), LogSigma, parmci(1,2), parmci(2,2), LognStats.df, TotalPhases(S), LognStats.chi2stat, LognP];
    AllGamStat(S,:) = [GamK, pci(1,1), pci(2,1), GamLambda, pci(1,2), pci(2,2), GamStats.df, TotalPhases(S), GamStats.chi2stat, GamP];

end


%=================== SUMMARY STATS
figure('name', 'Mean percept durations');
Means = nanmean(Durations);
Means = nanmedian(Durations);
SEMs = nanstd(Durations)./sqrt(TotalPhases);
DataHandle(1) = bar(Means);
hold on;
DataHandle(2) = errorbar(Means,SEMs,'.k');
set(gca, 'TickDir', 'Out',...
       	'box','off');
set(gca, 'xTickLabel', SubjectIDs);
ylabel('Mean percept duration (s)');


%================ GROUP NORMALIZED
figure('name','Group normalized percept durations');
NormDur = [];
for d = 1:5%numel(Durations(1,:))
    NormDur = [NormDur; Durations(:,d)/Means(d)];
end
NormDur = NormDur(~isnan(NormDur));
% hist(NormDur, 200);

BinSize = 0.05;
bins    = 0:BinSize:max(NormDur);
[n,xout] = hist(NormDur, bins);                                       
n = n/(numel(NormDur)*BinSize);                                       % For n = probability of a given phase duration 
h3 = bar(xout, n ,'histc');                                             
box off;
set(gca,'TickDir','out');
set(h3,'facecolor','w');
xlabel('Normalized percept duration');
ylabel('Frequency');
title('Patient group','fontsize',16);

hold on;
DurData = NormDur;

 %================== Fit LOGNORMAL duration distribution to data
    [parmhat, parmci] = lognfit(DurData);        	% 'pci' are confidence intervals (lower bound; upper bound)
    LogMu = parmhat(1);                            	% 'parmhat' returns estimated mu and sigma parameters
    LogSigma = parmhat(2);
    [LogMean,LogVar] = lognstat(LogMu,LogSigma);   	% obtain mean and variance for lognormal distribution
    logX = 0:0.02:Xlim(2);
    logPDF = lognpdf(logX,LogMu,LogSigma);
    logCDF = logncdf(logX,LogMu,LogSigma);

    %================== Fit GAMMA duration distribution to data
    [phat,pci] = gamfit2(DurData, alpha);         % 'phat' returns maximum likelihood estimates (MLEs), 'pci' are confidence intervals (lower bound; upper bound)
    GamK = phat(1);                                 % shape parameter (k)
    GamLambda = phat(2);                            % scale parameter (theta) 
    [GamMean,GamVar] = gamstat(GamK,GamLambda);     % obtain mean and variance for gamma distribution
    gamX = 0:0.02:Xlim(2);
    gamPDF = gampdf(gamX,GamK, GamLambda);
    gamCDF = gamcdf(gamX,GamK, GamLambda);

    %================== Fit GAMMA RATE distribution to data
    Rates = 1/DurData;
    [Rphat,Rpci] = gamfit2(Rates, alpha);           % Fit gamma distribution to SWITCH RATE data
    RGamK = Rphat(1);                               % shape parameter (k)
    RGamLambda = Rphat(2);                          % scale parameter (lambda) 
    [RGamMean,RGamVar] = gamstat(RGamK,RGamLambda); % obtain mean and variance for gamma rate distribution
    Xstep = 0.02;
    RgamX = 0:Xstep:(Xlim(2)/Xstep); 
    RategamPDF = gampdf(RgamX,RGamK, RGamLambda);
    RgamPDF = zeros(numel(RgamX),1);
    RgamCDF = zeros(numel(RgamX),1);
    for n = 1:numel(RgamX)
        X(n) = 1/(RgamX(n));
        RgamPDF(n) = (1/((RGamLambda^RGamK)*gamma(RGamK))) * X(n)^(RGamK-1) * exp(-X(n)/RGamLambda);
        RgamPDF(n) = RgamPDF(n)/(RgamX(n))^2;
        RgamCDF(n) = sum(RgamPDF);
    end
    RgamPDF = RgamPDF*30000;
    RgamX = RgamX*Xstep*2;

    %================== Plot all PDF fits
    PDF = plot(logX,logPDF, '-r', gamX,gamPDF, '-b', RgamX,RgamPDF, '-g', 'LineWidth', 2);
    set(gca,'xlim',[0 6]);
    
    
    
    
    
%% ====================== Box and whisker plot ============================
figure('name', 'Percept duration disributions');

bph = boxplot(DQDurations,'orientation','horizontal','notch','on','boxstyle','filled','plotstyle','compact','colors','rrrrrggbbbb');
set(gca,'ytick', 1:numel(SubjectIDs), 'yticklabel',SubjectIDs,'xlim',[0 30],'tickdir','out','xgrid','on');    
xlabel('Duration (s)');
s = AllStats(:,[5,6,7,8])';                                 % Get confidence intervals
set(bph(1,:),{'Xdata'},num2cell(s([1 4],:),1)');            % Modify whiskers
set(bph(2,:),{'Xdata'},num2cell(s([2 3],:),1)');            % Modify body
set(bph([3:7],:),{'Visible'},{'off'});                      
hold on
plot(AllStats(:,4)',1:numel(SubjectIDs), '+k');             % Plot medians


%% ========================= Fit parameter plot ===========================
figure('name','Distribution fit parameters');
subplot(1,2,1);
plot(AllLognStat(1:5,1), AllLognStat(1:5,4), 'dr');
hold on;
plot(AllLognStat(6:7,1), AllLognStat(6:7,4), 'og');
plot(AllLognStat(8:end,1), AllLognStat(8:end,4), 'ob');
xlabel('mu');
ylabel('sigma');
box off;
set(gca,'tickdir','out');
    
subplot(1,2,2);
plot(AllGamStat(1:5,1), AllGamStat(1:5,4), 'dr');
hold on;
plot(AllGamStat(6:7,1), AllGamStat(6:7,4), 'og');
plot(AllGamStat(8:end,1), AllGamStat(8:end,4), 'ob');
xlabel('k');
ylabel('lambda');
box off;
set(gca,'tickdir','out');
legend({'Patients','Older controls','Younger controls'});


PatientDurs = reshape(Durations(:,1:5),[1, numel(Durations(:,1:5))]);
ControlDurs = reshape(Durations(:,6:end),[1, numel(Durations(:,6:end))]);

PatientDurs(isnan(PatientDurs)) = [];
ControlDurs(isnan(ControlDurs)) = [];
[p,h,stats] = ranksum(PatientDurs,ControlDurs);